import 'dart:io';

import 'package:cache_store/cache_store.dart';
import 'package:cache_store/src/serializers/serializer.dart';
import 'package:cache_store/src/stores/file_store.dart';

Future<Cache<int>> update(String key) async {
  await Future.delayed(const Duration(seconds: 2));
  return Cache.fromNow(
    key: key,
    value: 42,
    expiresIn: const Duration(seconds: 1),
  );
}

Future main() async {
  final store = FileCacheStore<int>(
    file: File("test.cache"),
    serializer: CacheSerializer.builder(
      serializer: (v) => [v],
      deserializer: (data) => data.first,
    ),
  );

  var entry = await store.get("test");
  print("1. entry: $entry");

  entry = await store.get("test", ifExpired: update);
  print("2. entry: $entry");

  entry = await store.get("test", ifExpired: update);
  print("3. entry: $entry");

  entry = await store.get("other", ifExpired: update);
  print("3. entry: $entry");

  entry = await store.get("test", ifExpired: update);
  print("4. entry: $entry");

}
