import 'package:meta/meta.dart';

abstract class CacheSerializer<T> {
  List<int> serialize(T value);
  T deserialize(List<int> data);

  CacheSerializer();

  factory CacheSerializer.builder({
     @required ByteDeserializer<T> deserializer,
     @required ByteSerializer<T> serializer,
  }) {
    return _LambdaCacheSerializer(deserializer: deserializer, serializer: serializer);
  }
}

typedef T ByteDeserializer<T>(List<int> data);

typedef List<int> ByteSerializer<T>(T value);

class _LambdaCacheSerializer<T> extends CacheSerializer<T> {
  
  final ByteDeserializer<T> deserializer;

  final ByteSerializer<T> serializer;

  _LambdaCacheSerializer({@required this.deserializer, @required this.serializer});

  @override
  T deserialize(List<int> data) {
    return deserializer(data);
  }

  @override
  List<int> serialize(T value) {
    return serializer(value);
  }
  

}